********************************************************************************
*-- Name.......: PROC.PRG
*-- Programmer.: Kenneth J. Mayer, SofTech, Inc.
*-- Date.......: 05/01/91
*-- Version....: dBASE IV, 1.1
*-- Notes......: This is a generic procedure file with a bunch of routines from
*                a variety of sources. Anything I didn't write (most of what's
*                in this procedure file) was gotten from books, or the ATBBS, 
*                or elsewhere, and is always credited. 
********************************************************************************
*-- Note that to use this program file, the program must have the lines
*-- in it stating   SET PROCEDURE TO PROC  (usually early in the prog.)
*--                 DO SETCOLOR  && procedure to set screen colors
*--                 DO SETPRINT  && procedure for printer setup
*-- When done, it is a good idea to close the procedure file, using the
*-- command         SET PROCEDURE TO
*===============================================================================
* Quick Summary of Procedures/functions herein:
*===============================================================================
* Name       	Description
*----------  	-----------------------------------------------------------------
* setprint     PROCEDURE: Used to set the the appropriate default settings. 
*                 (Can be modified easily for other printers ...)
*              USAGE:  do setprint
*
* printerr     PROCEDURE: Used to display a printer error for STAND-ALONE
*                 systems. (The dBASE function PRINTSTATUS() doesn't work
*                 on a Network with Print Spoolers ...)
*              USAGE:  do setprint
*                      if .not. PRINTSTATUS()
*                         DO PRINTERR
*                      endif
*                             or
*                      DO WHILE .not. PRINTSTATUS()
*                         DO PRINTERR
*                      ENDDO
*
* setcolor     PROCEDURE: Used to set the screen colors for a system. It
*                 checks to see if a color monitor is attached (ISCOLOR()),
*                 and sets system variables, that can be used in SET COLOR OF
*                 commands.
*                   NOTE - if program CLRSHOW has been executed, it will create
*                   a file: COLOR.MEM that is used here. It can be used to
*                   design screen colors for the user, but if the COLOR.MEM
*                   file does not exist, this routine will use defaults
*                   given below. More docs on colors and SET commands
*                   are in the procedure's comments.
*              USAGE:  do setcolor
*
* extrclr      FUNCTION: Used to extract the first parameter of the MEMVARS
*                 created from SETCOLOR above. The SET COLOR OF commands can
*                 only use the first parameter. REQUIRES THAT SETCOLOR BE
*                 EXECUTED FIRST -- otherwise the memvars will not be accessible.
*              USAGE:  set color of highlight to &extrclr(cl_stand)
*
* open_screen  PROCEDURE: Used to give a texture to the background of the screen
*              USAGE:  do open_screen
*
* do_wait      FUNCTION: Used in place of the standard WAIT command, deals with
*                 centering the text at the message line (row 24).
*              USAGE:  lc_wait = do_wait("message")
*
* jazclear     PROCEDURE: Used to clear the screen from the middle out --
*                 could be used with open_screen, to really fancify things!
*              USAGE:  do jazclear
*
* center       PROCEDURE: Centers text on the screen with @says
*              USAGE:  DO CENTER with row,screenwidth,"color(s),"message"
*
* center2      FUNCTION: centers text similar to above, only two parameters.
*              USAGE:  @row,center2(ln_width,"TEXT") say "TEXT"
*                  or  @row,center2(ln_width,memvar) say memvar
*
* surround     FUNCTION: Displays a message surrounded by a box anywhere on 
*                 the screen
*              USAGE:  Dummy = SURROUND(row,column,"colors","text")
*
* message1     FUNCTION: Displays a message, centered, pauses until user 
*                 presses a key
*              USAGE:  Dummy = MESSAGE1(row,screenwidth,"colors","text")
*
* message2     FUNCTION: Displays a message in a window, pauses for user to 
*                 press key
*              USAGE:  Dummy = MESSAGE2("Text")
*
* message3     FUNCTION: Displays a message in a window, pauses for user, 
*                 will wrap a long message inside the window.
*              USAGE:  Dummy = MESSAGE3("text")
*
* message4     FUNCTION: Displays a 2-line message in a predefined window 
*                 and pauses
*              USAGE:  Dummy = MESSAGE4("text1","text2")
*
* monitor      PROCEDURE: Displays a status message to monitor a long-running 
*                 operation that operates on multiple records . . . 
*              USAGE:  DO MONITOR WITH "text"
*                      DO WHILE  (or SCAN)
*                         stuff -- process records
*                         @4,x DISPLAY ltrim(str(recno())) && current record
*                                        	         && in window MONITOR
*                      ENDDO  (endscan)
*                      DEACTIVATE WINDOW MONITOR
* 							  RELEASE WINDOW MONITOR
*                      RESTORE SCREEN FROM SC_MONITOR
*                      RELEASE SCREEN SC_MONITOR
*
* scrnhead     FUNCTION: Displays a heading on the screen in a box 2 
*                 spaces wider than the text, with a custom border (double 
*                 line top, single the rest)
*              USAGE: Dummy = SCRNHEAD("colors","Text")
*
* yesno        FUNCTION: Asks a yes/no question in a dialog window/box
*              USAGE: yesno(.t.,"Message1","Message2","Message3")
*                  can be used anywhere a function is used. 
*                  first parameter is default state (.t. or .f.)
*                  "Message1" is the first line of message (max length = 36)
*                    to display in the window describing the action
*                    There is a blank line between message1 and message2,
*                    but not between 2 and 3. You can leave any of them blank
*                    by placing a set of quotes there, i.e.:
*                           yesno(.t.,"Message1","","")
*
* yes          These two PROCEDUREs are a part of the yesno function above.
* no
*
* errormsg     FUNCTION: Display an error message: 1= ** ERROR **, 2 and 3
*                 are user defined, 4= Press any key to continue ...
*                 colors should be VIVID, since it's an error message.
*              USAGE: Dummy = ERRORMSG("message2","message3","Colors")
*                message2 and 3 should be 36 characters or less ...
*                see YESNO about blank message(s). Colors should include
*                foreground/background,foreground/background (border ...)
*
* datetext     FUNCTION: Display date in format Month, day year (eg July 1,1991)
*              USAGE:  DATETEXT(datefield)
*
* datetxt2     FUNCTION: Display date in format dayofweek, month day, year
*                                          (eg Monday, July 1, 1991)
*              USAGE:  DATETXT2(datefield)
*
* isunique     FUNCTION: Checks a keyfield to see if it is a unique entry
*              USAGE:  USE database ORDER tag ALIAS aliasname
*                      USE database ORDER tag ALIAS DupCheck AGAIN
*                         * second use of database is read only for ISUNIQUE
*                      @x,y SAY "prompt" GET lc_var PICTURE "picture";
*                           valid required ISUNIQUE(lc_var);
*                           message "Enter a UNIQUE code";
*                           error chr(7)+"Field must be unique!"
*                 where 'lc_var' is memory variable/field being checked against
*                 a specific 'field'. Make sure the correct index is set
*                 as this function uses the SEEK command.
*                 **** ALSO modify the field in the function below.
*
* alltrim      FUNCTION: Used to remove trailing/leading spaces
*              USAGE:  field = alltrim(field)
*
* shadow       PROCEDURE: Creates a shadow for a window (taken from the dBASE IV
*                 picklist commands)
*              USAGE:  SAVE SCREEN TO SC_name
*                      DEFINE WINDOW name FROM trow,tcol TO brow,bcol DOUBLE
*                      DO shadow WITH trow,tcol,brow,bcol
*                      ACTIVATE WINDOW name
*                            perform actions in window
*                      DEACTIVATE WINDOW name
*                      RESTORE SCREEN FROM SC_name
*                      RELEASE SCREEN SC_name
*
* fullwin      PROCEDURE: Overlays the menus on screen with a full window,
*                 so that processing is done in the window, and one can return
*                 directly to the menus, without redrawing screen and such.
*              USAGE:  DO fullwin WITH "colors","winname","screenname"
*                         perform whatever actions are needed in the window
*                      DEACTIVATE WINDOW winname
*                      RELEASE WINDOW winname
*                      RESTORE SCREEN FROM screenname
*                      RELEASE SCREEN screenname
*
* dosrun       FUNCTION: routine to run a DOS program, checks to see if a
*                 window is active -- if so, it avoids the inevitable
*                 "Press any key to continue" and the subsequent messing
*                 up of the screen display.
*              USAGE:  dummy = DOSRUN("doscommand /parm(s)...")    or
*                      dummy = DOSRUN(memvar)  && where memvar contains dos
*                                              && command and parameters ...
*
*===============================================================================
* END OF DESCRIPTIONS, On to the procedures and functions themselves!
*===============================================================================

PROCEDURE setprint && used to set the printer to the proper mode
	_pdriver  = "HPLAS2I"  && printer driver
	_ppitch   = "PICA" 	  && printer pitch (10 CPI)	
	_box      = .t.		  && make sure we can print boxes/line draw
	_ploffset = 0          && page offset (left side) to 0
	_lmargin  = 0          && left margin (also set to 0)
	_plength  = 60         && page length 60, since HP has top/bottom margins
	_peject   = "NONE"     && don't send extra blank pages . . .
	* _pquality = .t.        && set print quality to high -- not available
	                       && for HP Laserjets ...
	
RETURN   && -- end of procedure SETPRINT 

*---------------------------------------------------------------------

PROCEDURE printerr  && used to display specific printer error message
                    && This one's mine <g> Ken
	if iscolor()
		lc_color = "RG+/R,RG+/R,RG+/R"
	else
		lc_color = "N/W,N/W,N/W"
	endif
	
	define window printerr from  7,15 to 16,57 DOUBLE color &lc_color
	save screen to sc_perr     && store current screen
	do shadow with 7,15,16,57	&& shadow box!
	activate window printerr   && here we go ..
	
	@0,0 say chr(7) + "         *** PRINTER ERROR ***"
	@2,0 say          " The printer is not ready. Please check:"
	@3,0 say          "   1) that the printer is ON,"
	@4,0 say          "   2) that the printer is ONLINE, and"
	@5,0 say          "   3) that the printer has paper."
	@6,0 say          " "
	wait              " Press any key to continue . . ."
	
	deactivate window printerr
	release window printerr
	restore screen from sc_perr
	release screen sc_perr
	
RETURN  && from printerr

*---------------------------------------------------------------------

PROCEDURE setcolor && used to set the screen color memory variables
		** "Borrowed" from Phil Steele's PCSDEMO.PRG (public domain)
		** with lots of extra comments by me ...
	* note: colors can be combined in lots of interesting ways,
	* here are the color codes:
	* W  = White    W+  = Bright White
	* N  = Black    N+  = Grey
	* R  = Red      R+  = Pink
	* G  = Green    G+  = Light Green
	* B  = Blue     B+  = Light Blue
	* RG = Brown    RG+ = Yellow
	* RB = Magenta  RB+ = Bright Magenta
	* GB = Cyan     GB+ = Light Cyan
	*   NOTE that you can set the colors from CLRSHOW.PRG, which WILL
	*   store the correct memory variables (below) with the settings
	*   determined in that program into COLOR.MEM.
	*--------------------------------------------------------
	* Here are the SET COLOR OF options:
	*
	* SET COLOR OF NORMAL      TO
	*     @/Say without COLOR keyword
	*     Non-selected BROWSE items
	*     Layout design surface
	*     Uncolored box borders with @..TO
	*     Scoreboard
	* SET COLOR OF MESSAGES    TO
	*     Message Line messages
	*     Error messages in error box (dbase)
	*     Navigation line messages
	*     Available, unselected menu/list choices
	* SET COLOR OF TITLES      TO
	*     List/Display headings
	*     Help box headings
	*     BROWSE field name headings
	*     Ruler line (in editor)
	* SET COLOR OF BOX         TO
	*     Menu borders
	*     Pick List/prompt box
	* SET COLOR OF HIGHLIGHT   TO
	*     Highlighted menu/list choices
	* SET COLOR OF INFORMATION TO
	*     Clock/Help box/Status line
	* SET COLOR OF FIELDS      TO
	*     Selected fields in BROWSE
	*     Editable fields in @...Get
	* to use the memory variables (which all begin with CL_) defined
	* in this function, in the SET COLOR OF commands, you will need
	* the function EXTRCLR below this procedure to extract the first
	* paramter only. For example:
	*   SET COLOR OF BOX TO &extrclr(cl_warn)
	*    This will extract the first parameter of CL_Warn and use it
	*    to define the colors of boxes ...
	*----------------
	* These memvars can also be used in any command with a
	* COLOR clause, such as:
	*   DEFINE WINDOW name FROM x,y TO x2,y2 DOUBLE COLOR &cl_warn.
	*--------------------------------------------------------
	
	if file("COLOR.MEM")
		restore from color additive	&& if color.mem exists, restore from it
	else										&& otherwise, create it
		public cl_blank,cl_help,cl_data,cl_error,cl_entry,cl_stand,;
				 cl_menu,cl_warn
		C			= iscolor()				&& remember -- foreground/background
		cl_blank = "N/N,N/N,N"			&& black on black on black ...
		cl_func  = "N/W"
			* if iscolor() = true, define color, otherwise black/white
		cl_help  = iif(c,"N/G,G/N,N"      , "W+/N,N/W,N")   && help
		cl_data  = iif(c,"RG+/GB,GB/RG+,N", "W+/N,N/W,N")   && data entry fields
		cl_error = iif(c,"RG+/R,R/RG+,N"  , "W/N,N/W,N/W")  && error messages
		cl_entry = iif(c,"n/w,W/N,N"      , "n/w,W/N,N")    && data entry??
		cl_stand = iif(c,"w+/b,b/w,N"     , "w+/n,n/w,N")   && standard screen
		cl_menu  = iif(c,"rg+/b,B/RG+,N"  , "w+/n,N/W,N")   && menus
		cl_warn  = iif(c,"rg+/r,R/RG+,N"  , "w/n,N/W,N")    && warning messages
		save to color all like cl_*		&& create COLOR.MEM
	endif
	
	*-- change current color settings to these ...
	set color to &cl_stand	&& default
	
RETURN  && from procedure setcolor

*---------------------------------------------------------------------

FUNCTION EXTRCLR	&& extract the first color combination for use in
                  ** SET COLOR OF routines -- SETCOLOR must have been
                  ** executed first ...

	parameters lc_memvar
	
RETURN substr(lc_memvar,1,(at(",",lc_memvar)-1))

*---------------------------------------------------------------------

PROCEDURE open_screen

	** Designed to give textured opening screen -- written by Rick Price
	** stolen agregiously by Ken Mayer (with Rick's permission)
	
	clear
	x=0
	lc_Backdrp = chr(176)  && chr(176) = ""
	do while x<3
	   @x,0 to x+3,79 lc_Backdrp		&& display this box
	   sx=x
	   x=x+6
	   @x,0 to x+3,79 lc_Backdrp
	   x=x+6
	   @x,0 to x+3,79 lc_Backdrp
	   x=x+6
	   @x,0 to x+3,79 lc_Backdrp
	   x = sx+1
	enddo
	@24,0 to 24,79 lc_Backdrp

RETURN	&& end of procedure open_screen

*---------------------------------------------------------------------

FUNCTION Do_Wait

	** Another routine stolen from Rick Price to handle the need for
	** a wait, but killing the ESCAPE key, amongst other things.
	
	parameters lc_Message

	lc_WaitCur = SET("CURSOR")	&& save status of cursor
	SET CURSOR OFF
	
	** The the passed parameter (message_to_display) is null, use a generic
	** message.
	lc_Message = ;
	iif(""=lc_Message," Press any key to continue . . . ",lc_Message)
	* deal with centering/truncating message
	ln_MesLen = LEN(lc_Message)
	lc_Message = iif(ln_MesLen>80,LEFT(lc_Message,80),lc_Message)
	ln_MesLen = LEN(lc_Message)  && reset if message was longer than 80
	@24,INT((80-ln_MesLen)/2) say lc_Message 
	lc_RetStr=CHR(Inkey(0))
	set cursor &lc_waitcur

RETURN lc_RetStr  && end of function  Do_Wait

*---------------------------------------------------------------------

PROCEDURE JazClear

	** also stolen from Rick Price -- another fancy screen clear
	** explode outward from the center -- pretty fancy stuff
	
	ln_WinR1 = 0	&& row 1
	ln_WinR2 = 24  && row 2
	ln_WinC1 = 0   && column 1
	ln_WinC2 = 79  && column 2
	ln_Step = 1    && amount to increment by
	  * set starting point
	mn_WinC1 = INT((ln_WinC2-ln_WinC1)/2)+ln_WinC1
	mn_WinC2 = mn_WinC1+1
	mn_WinR1 = INT((ln_WinR2-ln_WinR1)/2)+ln_WinR1
	mn_WinR2 = mn_WinR1+1
	
	** Adjust step offset values: ln_ColOff & ln_RowOff
	** Vertical steps - mn_WinR1-ln_WinR1
	ln_TmpAdjR = int((ln_WinR2 - ln_WinR1)/2)
	ln_TmpAdjC = int((ln_WinC2 - ln_WinC1)/2)
	
	ln_AdjRow = ;
	iif(ln_TmpAdjC > ln_TmpAdjR, ln_TmpAdjR/ln_TmpAdjC,1) * ln_Step
	
	ln_AdjCol = ;
	iif(ln_TmpAdjR > ln_TmpAdjC, ln_TmpAdjC/ln_TmpAdjR,1) * ln_Step
	
	ln_colleft = ln_WinC1
	ln_colrite = ln_WinC2
	ln_RowTop = ln_WinR1
	ln_RowBot = ln_WinR2
	ln_WinC1 = mn_WinC1
	ln_WinC2 = mn_WinC2
	ln_WinR1 = mn_WinR1
	ln_WinR2 = mn_WinR2
	DO WHILE (ln_WinC1#ln_ColLeft .or. ln_WinC2#ln_ColRite .or. ;
		ln_WinR1 # ln_RowTop .or. ln_WinR2 # ln_RowBot)
		
		* Adjust coordinates for the clear (moving out from the middle)
		ln_WinR1 = ;
		ln_WinR1-IIF(ln_RowTop<ln_WinR1-ln_AdjRow,ln_AdjRow,ln_WinR1-ln_RowTop)
		ln_WinR2 = ;
		ln_WinR2+IIF(ln_RowBot>ln_WinR2+ln_AdjRow,ln_AdjRow,ln_RowBot-ln_WinR2)
		ln_WinC1 = ;
		ln_WinC1-IIF(ln_ColLeft<ln_WinC1-ln_AdjCol,ln_AdjCol,ln_WinC1-ln_ColLeft)
		ln_WinC2 = ;
		ln_WinC2+IIF(ln_ColRite>ln_WinC2+ln_AdjCol,ln_AdjCol,ln_ColRite-ln_WinC2)
		
		* Perform the clear
		@ln_WinR1,ln_WinC1 CLEAR TO ln_WinR2,ln_WinC2
		@ln_WinR1,ln_WinC1 TO ln_WinR2,ln_WinC2
	ENDDO
	CLEAR
RETURN   && from JazClear

*---------------------------------------------------------------------

PROCEDURE center
	** Used to center text on the screen with @Says, borrowed from
	** Miriam Liskin's dBASE IV Programming Book
	
	PARAMETERS ln_line,ln_width,lc_color,lc_text
	
	lc_text = alltrim(lc_text)	&& alltrim from below
	ln_col = (ln_width - len(lc_text)) /2
	@ln_line,ln_col say lc_text color &lc_color.
	
RETURN	&& from procedure center

*---------------------------------------------------------------------

FUNCTION center2   && centers text from a function, making it more
                   ** accessible than CENTER (above).
                   ** got the idea from Jeff (aka: The Musician/STUDENT
                   ** from the AT-Bulletin Board (ATBBS)).

	parameters ln_width,lc_text
	
RETURN (ln_width - len(lc_text)) / 2	&& end of function center2

*---------------------------------------------------------------------

FUNCTION Surround

	** from Miriam Liskin's Book
	** Displays text surrounded by a box anywhere on the screen
	** changed to a function: 4/19/91 KJM
	
	parameters ln_line,ln_column,lc_color,lc_text
	
	lc_text = " " + TRIM(lc_text) + " "	&& add spaces around text
	@ln_line-1,ln_column-1 to ln_line+1,ln_column+LEN(lc_text) DOUBLE;
		color &lc_color.  && draw box
	@ln_line,ln_column say lc_text color &lc_color.  && disp. text
	
RETURN "" && from procedure Surround

*---------------------------------------------------------------------

FUNCTION Message1

	** from Miriam Liskin's Book
	** Displays a centered message and pauses until user presses a key
	** uses CENTER above
	** changed to a function: 4/19/91 KJM
	
	parameters ln_line,ln_width,lc_color,lc_text
	
	@ln_line,0
	do center WITH ln_line,ln_width,lc_color,lc_text
	wait "" to x
	@ln_line,0

RETURN x && from procedure Message

*---------------------------------------------------------------------

FUNCTION Message2

	** from Miriam Liskin's Book
	** Displays a message in a window and pauses until user presses a key
	** changed to a function: 4/19/91 KJM
	** modified for shadow: 4/29/91 KJM
	
	parameters lc_text
	
	save screen to sc_message
	DEFINE Window Message FROM 10,10 to 14,70 DOUBLE COLOR &cl_warn.
	do shadow with 10,10,14,70
	ACTIVATE Window Message
	
	Do Center WITH 1,60,"W+/B",lc_text
	wait "" to x
	
	Deactivate Window Message
	Release Window Message
	restore screen from sc_message
	release screen sc_message

RETURN x && from Message2

*---------------------------------------------------------------------

FUNCTION message3

	** From Miriam Liskin's Book
	** displays message in a window and pauses until user presses a key
	** changed to a function: 4/19/91 KJM
	** modified to add shadow: 4/29/91 KJM
	
	Parameters lc_text
	
	ln_lines = Int(len(lc_text) / 38) + 5	&& set # of lines for window
	
	save screen to sc_message
	DEFINE WINDOW message FROM 8,20 to 8+ln_lines,60 DOUBLE COLOR &CL_WARN.
	do shadow with 8,20,8+ln_lines,60
	ACTIVATE WINDOW MESSAGE
	
	ln_lmargin = _lmargin
	ln_rmargin = _rmargin
	lc_alignment = _alignment
	ll_wrap = _wrap
	
	_lmargin = 1 
	_rmargin = 38
	_alignment = "CENTER"
	_wrap = .t.
	
	?lc_text
	?
	WAIT "    Press any key to continue . . ." to x
	
	_lmargin = ln_lmargin
	_rmargin = ln_rmargin
	_alignment = lc_alignment
	_wrap = ll_wrap
	
	deactivate window message
	release window message
	restore screen from sc_message
	release screen sc_message

RETURN x	&& from procedure Message3

*---------------------------------------------------------------------

FUNCTION message4

	** from Miriam Liskin's Book
	** Display a message in a predefined window and pause
	** changed to function: 4/19/91 KJM
	** modified to add shadow: 4/29/91 KJM
	
	parameters lc_text1,lc_text2
	
	save screen to sc_message
	define window MONITOR from 10,10 to 17,70 double COLOR &cl_warn.
	do shadow with 10,10,17,70
	activate window MONITOR
	
	ln_lmargin = _lmargin
	ln_rmargin = _rmargin
	ll_wrap = _wrap

	_lmargin = 1 
	_rmargin = 58
	_wrap = .t.
	
	do center with 1,58,"",lc_text1
	do center with 2,58,"",lc_text2
	do center with 4,58,"","Press any key to continue . . ."
	wait "" to x

	_lmargin = ln_lmargin
	_rmargin = ln_rmargin
	_wrap = ll_wrap
	
	deactivate window MONITOR
	release window MONITOR
	restore screen from sc_message
	release screen sc_message
	
RETURN x	&& from procedure MESSAGE4

*---------------------------------------------------------------------

PROCEDURE monitor

	** taken from Miriam Liskin's Book
	** display a status message to monitor a long-running operation
	** user must specify in processing the record# to place in the
	** box on line 4 . . . Must also deactivate window MONITOR and
	** release it from MEMORY
	** Modified to add shadow: 4/29/91 KJM
	
	parameters lc_text
	
	save screen to sc_monitor
	define window MONITOR From 10,10 to 18,70 DOUBLE COLOR &cl_help.
	do shadow with 10,10,18,70
	activate window monitor
	
	do center with 1,60,"",lc_text
	do center with 2,60,"","Please do not interrupt"
	@4,10 say "Working on record          of " + ltrim(str(reccount(),5))
	
	** remember to: DEACTIVATE WINDOW MONITOR
	**              RELEASE WINDOW MONITOR
	**              RESTORE SCREEN FROM SC_MONITOR
	**              RELEASE SCREEN SC_MONITOR
	
RETURN	&& from procedure MONITOR

*---------------------------------------------------------------------

FUNCTION scrnhead

	** taken from Miriam Liskin's Book
	** Display a heading in a box 2 spaces wider than text with
	** custom border (double line top, single line sides)
	
	parameters lc_color,lc_text
	
	lc_text = " "+TRIM(lc_text)+" "    && ad spaces to left and right
	lc_textstart = (80-len(trim(lc_text)))/2
	@1,lc_textstart-1 to 3,81-lc_textstart 205,196,179,179,213,184,192,217;
		color &lc_color.	&& display box
	@2, lc_textstart say lc_text color &lc_color. && display text

RETURN ""	&& from procedure scrnhead

*---------------------------------------------------------------------

FUNCTION Yesno

	** from Miriam Liskin's Book, changed to a function
	** asks a Yes-No question in a dialog box!
	** Modified to function: 4/19/91 KJM
	** Modified to shadow window: 4/29/91 KJM
	
	parameter ll_answer,lc_mess1,lc_mess2,lc_mess3
	
	save screen to sc_yesno
	DEFINE Window yesno FROM 8,20 to 16,60 double color &cl_help.
	
	define menu yesno
	define pad yes of yesno Prompt "[Yes]" at 5,10
	define pad no  of yesno Prompt "[No]"  at 5,25
	On Selection Pad Yes of yesno do yes
	On Selection pad No  of yesno do no
	
	do shadow with 8,20,16,60
	ACTIVATE Window yesno
	ln_lmargin = _lmargin	&& store system values
	ln_rmargin = _rmargin
	ll_wrap    = _wrap
	_lmargin = 2			&& set local values
	_rmargin = 38
	_wrap    = .t.
	
	do center with 0,38,"",lc_mess1		&& center the text
	do center with 2,38,"",lc_mess2
	do center with 3,38,"",lc_mess3
	if ll_answer
		ACTIVATE MENU yesno PAD Yes
	else
		ACTIVATE MENU yesno PAD No
	endif
	
	_lmargin = ln_lmargin	&& reset system values
	_rmargin = ln_rmargin
	_wrap    = ll_wrap
	
	Deactivate Window yesno
	release window yesno
	restore screen from sc_yesno
	release screen sc_yesno
	release menu yesno

RETURN ll_answer && from procedure Yesno

PROCEDURE Yes
	ll_answer = .t.
	Deactivate Menu
RETURN

PROCEDURE No
	ll_answer = .f.
	Deactivate Menu
RETURN

*---------------------------------------------------------------------

FUNCTION errormsg	&& display a window describing an error
	** this one's mine (Ken Mayer)
	
	PARAMETERS lc_mess1,lc_mess2,lc_color
	
	save screen to sc_err
	define window ERR from 8,20 to 15,60 double color &lc_color.
	do shadow with 8,20,15,60
	activate window ERR
	
	do center with 0,38,"","** ERROR **"
	do center with 2,38,"",lc_mess1
	do center with 3,38,"",lc_mess2
	do center with 5,38,"","Press any key to continue ..."
	lc_wait=inkey(0)
	
	deactivate window ERR
	release window ERR
	restore screen from sc_err
	release screen sc_err
	
RETURN lc_wait	&& from function ERRORMSG

*---------------------------------------------------------------------

FUNCTION Datetext

	** stolen from Miriam Liskin's book
	** displays date in text format (e.g., July 1, 1991)
	
	parameters ld_date
	
RETURN CMONTH(ld_date)+" "+ltrim(str(day(ld_date),2))+", "+;
		str(year(ld_date),4)

*---------------------------------------------------------------------

FUNCTION datetxt2

	** from Miriam Liskin's book
	** displays date in text format (e.g., Monday, July 1, 1991)
	
	parameters ld_date
	
RETURN CDOW(ld_date)+", "+cmonth(ld_date)+" "+;
		 ltrim(str(day(ld_date),2))+", "+str(year(ld_date),4)

*---------------------------------------------------------------------

FUNCTION Isunique

	** from Miriam Liskin, minor mods by Rick Price
	** Used to determine if a keyfield is unique
		
	parameters lc_var
		
	ln_record = recno()	&& store current record number
	ll_unique = .t.		&& init to true
	lc_dbf = alias()	&& store current alias, so we can return to it
	SELECT DupCheck	&& second copy of database	
	GO TOP            && ensure we're at the top ...
		
	SEEK lc_var		&& make sure database is set to correct index here
	locate for keyfield = lc_var .and. recno() <> ln_record REST
*		        ^^^^^^^^                               *
* ========>	MUST BE KEYFIELD IN DATABASE <========== *
	if found()
		ll_unique = .f.
	endif
		
	SELECT (lc_dbf)		&& back to original copy of file
	
RETURN ll_unique		&& return value of that field

*---------------------------------------------------------------------

FUNCTION alltrim  && trim leading/trailing spaces
	
	* Borrowed from Phil Steele's PCSDEMO.PRG (public domain)
	
	PARAMETERS lc_string
	
RETURN ltrim(rtrim(lc_string))  && that's it.

*---------------------------------------------------------------------

PROCEDURE shadow  && routine is taken from A-T routines generated by
                  ** CCBooster to do a shadow on picklist ...

	parameters ln_x1,ln_y1,ln_x2,ln_Y2

	ln_x0 = ln_x2+1
	ln_y0 = ln_y2+2
	ln_dx = 1
	ln_dy = (ln_y2-ln_y1) / (ln_x2-ln_x1)
	DO WHILE ln_x0 <> ln_x1 .or. ln_y0 <> ln_y1+2
		@ ln_x0,ln_y0 FILL TO ln_x2+1,ln_y2+2 COLOR n+/n
		ln_x0 = IIF(ln_x0<>ln_x1,ln_x0 - ln_dx,ln_x0)
		ln_y0 = IIF(ln_y0<>ln_y1+2,ln_y0 - ln_dy,ln_y0)
		ln_y0 = IIF(ln_y0<ln_y1+2,ln_y1+2,ln_y0)
	ENDDO
	
RETURN  && from procedure SHADOW

*---------------------------------------------------------------------

PROCEDURE fullwin  && just a small routine to setup a window overlaying
						 ** the main/calling screen ... completely (except
						 ** for row 24, which is used for messages)
	
	parameters lc_color,lc_winname,sc_screen
	
	define window &lc_winname from 0,0 to 23,79 NONE color &lc_color.
	save screen to &sc_screen.
	activate window &lc_winname.
	
	* see documentation at beginning of procedure file about releasing
	* and deactivating things ...
	
RETURN  && from procedure fullwin

*---------------------------------------------------------------------

FUNCTION dosrun  && function created by Micheal P. Dean, posted on the
                 ** Ashton-Tate BBS, used to avoid problems with active
                 ** windows and DOS commands executed from within progs.

	PARAMETER lc_cmd
	
	lc_window = WINDOW()
	if len(trim(lc_window)) > 0
		DEACTIVATE WINDOW &lc_window
	endif
	SET CONSOLE OFF
	RUN &lc_cmd
	SET CONSOLE ON
	if len(trim(lc_window)) > 0
		ACTIVATE WINDOW &lc_window
	endif
	
RETURN ""  && from function DOSRUN

*---------------------------------------------------------------------
* End of procedure File
*---------------------------------------------------------------------
